#include "lnp.h"

/*
 * formats the request message,
 * if return value is greater or equal to buffers size
 * then the buffer was truncated and thus not null-terminated
 */
int makeMessage(char *buffer, int len, char *hostname)
{
  return snprintf(buffer, len, "%s\n%s\n", version, hostname);  
}
/*
 * if the protocol versions do not match, 
 * or the buffer is shorter than version number
 * -1 is returned
 */
int checkProtocolVersion(char *buffer, int len)
{
  if(vlen > len || strncmp(version, buffer, vlen) != 0)
  {
    errno = EINVAL;
    return -1;
  }
  else return 0;
}
/* 
 * format check, the protocol has two lines
 * of character data, both end with a single line-feed
 * return -1 if format is violated
 */
int checkProtocolFormat(char *buffer, int len)
{
  unsigned lf1, lf2;
  lf1 = 0;
  while( (lf1 < len) && (*(buffer+lf1) != '\n') ) lf1++;
  lf2 = lf1+1;
  while( (lf2 < len) && (*(buffer+lf2) != '\n') ) lf2++;

  if( lf1 >= lf2 || lf2 >= len )
  {
    errno = EINVAL;
    return -1;
  }

  return 0;
}

