#include "lnp.h"

/*
 * Timeout handler
 */
void sighandler(int sig)
{
  char *line = "No host responded to that name.\n";
  write(STDOUT_FILENO, line, strlen(line));
  exit(EXIT_SUCCESS);
}


int main(int argc, char *argv[])
{
  /* check hostname argument */
  if( argc < 2 ) error(EXIT_FAILURE, EINVAL, "Aufruf: %s <hostname>", argv[0]); 

  /* initialize udp socket */
  int udp;
  if( (udp = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) perror("socket"); 

  /* enable broadcast socket */
  int opt = 1;
  if( setsockopt(udp, SOL_SOCKET, SO_BROADCAST, &opt, sizeof(opt)) == -1) perror("set broadcast socket");

  /* define destination broadcast address and port */
  struct sockaddr_in inaddr;
  inaddr.sin_port = htons(2019);
  inaddr.sin_family = AF_INET;
  inaddr.sin_addr.s_addr = INADDR_BROADCAST;

  /* allocate protocol/packet buffer */
  char buffer[BUFSIZE+vlen+2];
  socklen_t anz, addrlen;

  /* prepare address for answer/reply */
  struct sockaddr_in sender = {0};

  /* format request message udp packet */
  int len = makeMessage(buffer, sizeof(buffer), argv[1]);
  if( len >= sizeof(buffer) )
  {
    error(0, EINVAL, "buffer too small, truncated");
    len = sizeof(buffer);
  }

  /* send request message/packet */
  if(sendto(udp, buffer, len, 0, (const struct sockaddr*) &inaddr, sizeof(inaddr)) == -1) perror("sendto");

  /* set timeout, in case of no answer -> exits program */
  signal(SIGALRM, sighandler);
  alarm(1);

  /* receive a possible answer/reply message */
  if( (anz = recvfrom(udp, buffer, sizeof(buffer), 0, (struct sockaddr*) &sender, &addrlen)) == -1) perror("recvfrom"); 

  /* null-terminate buffer (optional) */  
  if( anz < sizeof(buffer)-1 ) buffer[anz] = '\0';  
  else buffer[sizeof(buffer)-1] = '\0';

  /* check reply message format */
  if( checkProtocolFormat(buffer, sizeof(buffer)) ) error(EXIT_FAILURE, errno, "packet format violation");
  /* check reply message version */
  if( checkProtocolVersion(buffer, sizeof(buffer)) ) error(0, errno, "%s version mismatch: %s", version, buffer);

  /* print the ip-address -> only correct, if version-length identical */
  printf("%s", buffer+vlen+1);

  /* clean up and exit */
  close(udp);
  return EXIT_SUCCESS;

}

