#include "lnp.h"

/* Name-length of the interface name */
#define IFACE_MAX 32

int main(int argc, char *argv[])
{
  /* name of the interface, which ip shall be requested */
  char iface[IFACE_MAX] = {"eth0"};

  /* the interface name can be given on command-line */
  signed char option;
  while( (option = getopt(argc, argv, "i:")) > 0 )
  {
    switch(option)
    {
      case 'i':
	if( snprintf(iface, IFACE_MAX, optarg) >= IFACE_MAX) 
	{
	  perror("interface name truncated");
	  iface[IFACE_MAX-1] = '\0';
	}
    }
  }

  /* initialize the server side udp socket */
  int udp;
  if( (udp = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) perror("socket"); 

  /* set broadcast option to allow receiving broadcasts */
  int opt = 1;
  if( setsockopt(udp, SOL_SOCKET, SO_BROADCAST, &opt, sizeof(opt)) == -1) perror("setsockopt");

  /* define endpoint address and port for listening */
  struct sockaddr_in inaddr;
  inaddr.sin_port = htons(2019);
  inaddr.sin_family = AF_INET;
  inaddr.sin_addr.s_addr = INADDR_ANY;

  /* bind socket to that endpoint (here: all interfaces) */
  if( bind(udp, (struct sockaddr*) &inaddr, sizeof(inaddr)) == -1 ) perror("bind");

  /* initialize message buffer for sending/receiving */
  char buffer[BUFSIZE+vlen+2];
  /* initialize socket address for replying */
  socklen_t anz, addrlen;
  struct sockaddr_in sender = {0};
  addrlen = sizeof(struct sockaddr_in);

  /* find out the name of this host for comparison */
  char hostname[BUFSIZE] = {0};
  if( gethostname(hostname, sizeof(hostname)) == -1 )
  {
     perror("gethostame");
     hostname[64] = '\0';
  }

  /* preparation done */
  printf("lnpd running on host: %s using interface: %s\n", hostname, iface);

  while(1)
  { 
    /* wait for incoming message */
    if((anz = recvfrom(udp, buffer, sizeof(buffer), 0, (struct sockaddr*) &sender, &addrlen)) == -1) perror("recv");

    /* check if this message has the right format, if not -> discard */
    if( checkProtocolFormat(buffer, sizeof(buffer)) ) {  error(0, errno, "packet format violation"); continue; }

    /* check if the message has the correct protocol version */
    if(checkProtocolVersion(buffer, vlen)) error(0, errno, "%s version mismatch: %s", version, buffer);

    /* compare the hostname with the one requested */
    if(strncmp(hostname, buffer+vlen+1, strlen(hostname)) == 0)
    {
      /* output that there is a match */
      printf("message from %s on port %d\n", inet_ntoa(sender.sin_addr), ntohs(sender.sin_port)); 
      write(STDOUT_FILENO, buffer+vlen+1, anz-vlen-1);
    
      /* 
       * get the ip-address of the desired interface 
       * (could change during runtime, e.g. through DHCP) 
       */
      struct ifaddrs *ifaddr;
      if( getifaddrs(&ifaddr) == -1 ) perror("getifaddrs");   

      for(struct ifaddrs *rp = ifaddr; rp != NULL; rp = rp->ifa_next)
      {
        /* filter the desired interface name */
        if( rp->ifa_addr->sa_family == AF_INET && strncmp(iface, rp->ifa_name, 4) == 0 )
        {
            /* output the dedicated ip-address and reply a message to the caller */
            printf("%s is on %s with addr %s\n", hostname, rp->ifa_name, inet_ntoa(((struct sockaddr_in*)(rp->ifa_addr))->sin_addr));
            if( makeMessage(buffer, sizeof(buffer), inet_ntoa(((struct sockaddr_in*)(rp->ifa_addr))->sin_addr)) >= sizeof(buffer))
            {
              perror("addr truncated");
              buffer[sizeof(buffer)-1] = '\0';
            }
            if( sendto(udp, buffer, strlen(buffer), 0, (const struct sockaddr*) &sender, addrlen) == -1) perror("sendto"); 
        }
      }
      /* getifaddrs dynamically allocated memory, so it has to be freed before the next run */
      freeifaddrs(ifaddr);
    }
  } 

  /* the loop has finished, now cleanup and exit */
  close(udp);
  return EXIT_SUCCESS;

}

